import actionCreatorFactory from "typescript-fsa";
import { useSelector } from "react-redux";
import { reducerWithInitialState } from "typescript-fsa-reducers";
export const cellInitialState = {
    source: '',
    outputsCount: -1,
    kernelAvailable: false,
};
/* Selectors */
export const selectCell = () => useSelector((state) => {
    if (state.cell) {
        return state.cell;
    }
    return cellInitialState;
});
/* Actions */
export var ActionType;
(function (ActionType) {
    ActionType["SOURCE"] = "cell/SOURCE";
    ActionType["OUTPUTS_COUNT"] = "cell/OUTPUTS_COUNT";
    ActionType["EXECUTE"] = "cell/EXECUTE";
    ActionType["UPDATE"] = "cell/UPDATE";
})(ActionType || (ActionType = {}));
const actionCreator = actionCreatorFactory('jupyterReact');
export const cellActions = {
    source: actionCreator(ActionType.SOURCE),
    outputsCount: actionCreator(ActionType.OUTPUTS_COUNT),
    execute: actionCreator(ActionType.EXECUTE),
    update: actionCreator(ActionType.UPDATE),
};
/* Reducers */
export const cellReducer = reducerWithInitialState(cellInitialState)
    .case(cellActions.execute, (state, payload) => {
    if (state.adapter) {
        state.adapter.execute();
    }
    return Object.assign({}, state);
})
    .case(cellActions.source, (state, source) => {
    return Object.assign(Object.assign({}, state), { source });
})
    .case(cellActions.update, (state, partial) => (Object.assign(Object.assign({}, state), partial)))
    .case(cellActions.outputsCount, (state, outputsCount) => {
    return Object.assign(Object.assign({}, state), { outputsCount });
});
