var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { sessionContextDialogs } from '@jupyterlab/apputils';
import { NotebookActions, NotebookSearchProvider } from '@jupyterlab/notebook';
import { SearchDocumentModel, SearchDocumentView } from '@jupyterlab/documentsearch';
import { Widget } from '@lumino/widgets';
import { nullTranslator } from '@jupyterlab/translation';
export const cmdIds = {
    invoke: 'completer:invoke',
    select: 'completer:select',
    invokeNotebook: 'completer:invoke-notebook',
    selectNotebook: 'completer:select-notebook',
    startSearch: 'documentsearch:start-search',
    findNext: 'documentsearch:find-next',
    findPrevious: 'documentsearch:find-previous',
    save: 'notebook:save',
    interrupt: 'notebook:interrupt-kernel',
    restart: 'notebook:restart-kernel',
    switchKernel: 'notebook:switch-kernel',
    runAndAdvance: 'notebook-cells:run-and-advance',
    run: 'notebook:run-cell',
    runAll: 'notebook:run-all-cells',
    deleteCells: 'notebook-cells:delete',
    insertAbove: 'notebook-cells:insert-above',
    insertBelow: 'notebook-cells:insert-below',
    moveUp: 'notebook-cells:move-up',
    moveDown: 'notebook-cells:move-down',
    selectAbove: 'notebook-cells:select-above',
    selectBelow: 'notebook-cells:select-below',
    extendAbove: 'notebook-cells:extend-above',
    extendTop: 'notebook-cells:extend-top',
    extendBelow: 'notebook-cells:extend-below',
    extendBottom: 'notebook-cells:extend-bottom',
    editMode: 'notebook:edit-mode',
    merge: 'notebook-cells:merge',
    split: 'notebook-cells:split',
    commandMode: 'notebook:command-mode',
    undo: 'notebook-cells:undo',
    redo: 'notebook-cells:redo'
};
export const NotebookCommands = (commands, nbWidget, handler) => {
    commands.addCommand(cmdIds.invoke, {
        label: 'Completer: Invoke',
        execute: () => handler.invoke()
    });
    commands.addCommand(cmdIds.select, {
        label: 'Completer: Select',
        execute: () => handler.completer.selectActive()
    });
    commands.addCommand(cmdIds.invokeNotebook, {
        label: 'Invoke Notebook',
        execute: () => {
            var _a;
            if (((_a = nbWidget.content.activeCell) === null || _a === void 0 ? void 0 : _a.model.type) === 'code') {
                return commands.execute(cmdIds.invoke);
            }
        }
    });
    commands.addCommand(cmdIds.selectNotebook, {
        label: 'Select Notebook',
        execute: () => {
            var _a;
            if (((_a = nbWidget.content.activeCell) === null || _a === void 0 ? void 0 : _a.model.type) === 'code') {
                return commands.execute(cmdIds.select);
            }
        }
    });
    commands.addCommand(cmdIds.save, {
        label: 'Save',
        execute: () => nbWidget.context.save()
    });
    let searchInstance;
    commands.addCommand(cmdIds.startSearch, {
        label: 'Find…',
        execute: () => {
            if (!searchInstance) {
                const provider = new NotebookSearchProvider(nbWidget, nullTranslator);
                const searchModel = new SearchDocumentModel(provider, 500);
                searchInstance = new SearchDocumentView(searchModel);
                /**
                 * Activate the target widget when the search panel is closing
                 */
                searchInstance.closed.connect(() => {
                    if (!nbWidget.isDisposed) {
                        nbWidget.activate();
                    }
                });
                searchInstance.disposed.connect(() => {
                    if (!nbWidget.isDisposed) {
                        nbWidget.activate();
                    }
                    // find next and previous are now disabled
                    commands.notifyCommandChanged();
                });
                /**
                 * Dispose resources when the widget is disposed.
                 */
                nbWidget.disposed.connect(() => {
                    searchInstance === null || searchInstance === void 0 ? void 0 : searchInstance.dispose();
                    searchModel.dispose();
                    provider.dispose();
                });
            }
            if (!searchInstance.isAttached) {
                Widget.attach(searchInstance, nbWidget.node);
                searchInstance.node.style.top = `${nbWidget.toolbar.node.getBoundingClientRect().height +
                    nbWidget.contentHeader.node.getBoundingClientRect().height}px`;
                if (searchInstance.model.searchExpression) {
                    searchInstance.model.refresh();
                }
            }
            searchInstance.focusSearchInput();
        }
    });
    commands.addCommand(cmdIds.findNext, {
        label: 'Find Next',
        isEnabled: () => !!searchInstance,
        execute: () => __awaiter(void 0, void 0, void 0, function* () {
            if (!searchInstance) {
                return;
            }
            yield searchInstance.model.highlightNext();
        })
    });
    commands.addCommand(cmdIds.findPrevious, {
        label: 'Find Previous',
        isEnabled: () => !!searchInstance,
        execute: () => __awaiter(void 0, void 0, void 0, function* () {
            if (!searchInstance) {
                return;
            }
            yield searchInstance.model.highlightPrevious();
        })
    });
    commands.addCommand(cmdIds.interrupt, {
        label: 'Interrupt',
        execute: () => __awaiter(void 0, void 0, void 0, function* () { var _a, _b; return (_b = (_a = nbWidget.context.sessionContext.session) === null || _a === void 0 ? void 0 : _a.kernel) === null || _b === void 0 ? void 0 : _b.interrupt(); })
    });
    commands.addCommand(cmdIds.restart, {
        label: 'Restart Kernel',
        execute: () => sessionContextDialogs.restart(nbWidget.context.sessionContext)
    });
    commands.addCommand(cmdIds.switchKernel, {
        label: 'Switch Kernel',
        execute: () => sessionContextDialogs.selectKernel(nbWidget.context.sessionContext)
    });
    commands.addCommand(cmdIds.runAndAdvance, {
        label: 'Run and Advance',
        execute: () => {
            return NotebookActions.runAndAdvance(nbWidget.content, nbWidget.context.sessionContext);
        }
    });
    commands.addCommand(cmdIds.run, {
        label: 'Run',
        execute: () => {
            return NotebookActions.run(nbWidget.content, nbWidget.context.sessionContext);
        }
    });
    commands.addCommand(cmdIds.runAll, {
        label: 'Run All Cells',
        execute: () => {
            return NotebookActions.runAll(nbWidget.content, nbWidget.context.sessionContext);
        }
    });
    commands.addCommand(cmdIds.deleteCells, {
        label: 'Delete',
        execute: () => {
            return NotebookActions.deleteCells(nbWidget.content);
        }
    });
    commands.addCommand(cmdIds.insertAbove, {
        label: 'Insert Above',
        execute: () => {
            return NotebookActions.insertAbove(nbWidget.content);
        }
    });
    commands.addCommand(cmdIds.insertBelow, {
        label: 'Insert Below',
        execute: () => {
            return NotebookActions.insertBelow(nbWidget.content);
        }
    });
    commands.addCommand(cmdIds.moveUp, {
        label: 'Insert Below',
        execute: () => {
            return NotebookActions.moveUp(nbWidget.content);
        }
    });
    commands.addCommand(cmdIds.moveDown, {
        label: 'Insert Below',
        execute: () => {
            return NotebookActions.moveDown(nbWidget.content);
        }
    });
    commands.addCommand(cmdIds.editMode, {
        label: 'Edit Mode',
        execute: () => {
            nbWidget.content.mode = 'edit';
        }
    });
    commands.addCommand(cmdIds.commandMode, {
        label: 'Command Mode',
        execute: () => {
            nbWidget.content.mode = 'command';
        }
    });
    commands.addCommand(cmdIds.selectBelow, {
        label: 'Select Below',
        execute: () => NotebookActions.selectBelow(nbWidget.content)
    });
    commands.addCommand(cmdIds.selectAbove, {
        label: 'Select Above',
        execute: () => NotebookActions.selectAbove(nbWidget.content)
    });
    commands.addCommand(cmdIds.extendAbove, {
        label: 'Extend Above',
        execute: () => NotebookActions.extendSelectionAbove(nbWidget.content)
    });
    commands.addCommand(cmdIds.extendTop, {
        label: 'Extend to Top',
        execute: () => NotebookActions.extendSelectionAbove(nbWidget.content, true)
    });
    commands.addCommand(cmdIds.extendBelow, {
        label: 'Extend Below',
        execute: () => NotebookActions.extendSelectionBelow(nbWidget.content)
    });
    commands.addCommand(cmdIds.extendBottom, {
        label: 'Extend to Bottom',
        execute: () => NotebookActions.extendSelectionBelow(nbWidget.content, true)
    });
    commands.addCommand(cmdIds.merge, {
        label: 'Merge Cells',
        execute: () => NotebookActions.mergeCells(nbWidget.content)
    });
    commands.addCommand(cmdIds.split, {
        label: 'Split Cell',
        execute: () => NotebookActions.splitCell(nbWidget.content)
    });
    commands.addCommand(cmdIds.undo, {
        label: 'Undo',
        execute: () => NotebookActions.undo(nbWidget.content)
    });
    commands.addCommand(cmdIds.redo, {
        label: 'Redo',
        execute: () => NotebookActions.redo(nbWidget.content)
    });
    const bindings = [
        {
            selector: '.jp-Notebook.jp-mod-editMode .jp-mod-completer-enabled',
            keys: ['Tab'],
            command: cmdIds.invokeNotebook
        },
        {
            selector: `.jp-mod-completer-active`,
            keys: ['Enter'],
            command: cmdIds.selectNotebook
        },
        {
            selector: '.jp-Notebook',
            keys: ['Ctrl Enter'],
            command: cmdIds.run
        },
        {
            selector: '.jp-Notebook',
            keys: ['Shift Enter'],
            command: cmdIds.runAndAdvance
        },
        {
            selector: '.jp-Notebook',
            keys: ['Accel S'],
            command: cmdIds.save
        },
        {
            selector: '.jp-Notebook',
            keys: ['Accel F'],
            command: cmdIds.startSearch
        },
        {
            selector: '.jp-Notebook',
            keys: ['Accel G'],
            command: cmdIds.findNext
        },
        {
            selector: '.jp-Notebook',
            keys: ['Accel Shift G'],
            command: cmdIds.findPrevious
        },
        {
            selector: '.jp-Notebook.jp-mod-commandMode:focus',
            keys: ['I', 'I'],
            command: cmdIds.interrupt
        },
        {
            selector: '.jp-Notebook.jp-mod-commandMode:focus',
            keys: ['0', '0'],
            command: cmdIds.restart
        },
        {
            selector: '.jp-Notebook.jp-mod-commandMode:focus',
            keys: ['Enter'],
            command: cmdIds.editMode
        },
        {
            selector: '.jp-Notebook.jp-mod-editMode',
            keys: ['Escape'],
            command: cmdIds.commandMode
        },
        {
            selector: '.jp-Notebook.jp-mod-commandMode:focus',
            keys: ['Shift M'],
            command: cmdIds.merge
        },
        {
            selector: '.jp-Notebook.jp-mod-editMode',
            keys: ['Ctrl Shift -'],
            command: cmdIds.split
        },
        {
            selector: '.jp-Notebook.jp-mod-commandMode:focus',
            keys: ['J'],
            command: cmdIds.selectBelow
        },
        {
            selector: '.jp-Notebook.jp-mod-commandMode:focus',
            keys: ['ArrowDown'],
            command: cmdIds.selectBelow
        },
        {
            selector: '.jp-Notebook.jp-mod-commandMode:focus',
            keys: ['K'],
            command: cmdIds.selectAbove
        },
        {
            selector: '.jp-Notebook.jp-mod-commandMode:focus',
            keys: ['ArrowUp'],
            command: cmdIds.selectAbove
        },
        {
            selector: '.jp-Notebook.jp-mod-commandMode:focus',
            keys: ['Shift K'],
            command: cmdIds.extendAbove
        },
        {
            selector: '.jp-Notebook.jp-mod-commandMode:focus',
            keys: ['Shift J'],
            command: cmdIds.extendBelow
        },
        {
            selector: '.jp-Notebook.jp-mod-commandMode:focus',
            keys: ['Z'],
            command: cmdIds.undo
        },
        {
            selector: '.jp-Notebook.jp-mod-commandMode:focus',
            keys: ['Y'],
            command: cmdIds.redo
        }
    ];
    bindings.map(binding => commands.addKeyBinding(binding));
};
export default NotebookCommands;
