import { useSelector } from "react-redux";
import actionCreatorFactory from "typescript-fsa";
import { combineEpics } from "redux-observable";
import { ignoreElements, tap } from "rxjs/operators";
import { ofAction } from "@datalayer/typescript-fsa-redux-observable";
import { reducerWithInitialState } from "typescript-fsa-reducers";
import { cmdIds } from "./NotebookCommands";
export const notebookInitialState = {
    activeCell: undefined,
    kernelStatus: undefined,
    notebookChange: undefined,
    portals: [],
};
/* Selectors */
export const selectNotebook = () => useSelector((state) => {
    if (state.notebook) {
        return state.notebook;
    }
    return notebookInitialState;
});
/* Actions */
export var ActionType;
(function (ActionType) {
    ActionType["UPDATE"] = "notebook/UPDATE";
    ActionType["NOTEBOOK_CHANGE"] = "notebook/NOTEBOOK_CHANGE";
    ActionType["ACTIVE_CELL_CHANGE"] = "notebook/ACTIVE_CELL_CHANGE";
    ActionType["KERNEL_STATUS_CHANGE"] = "notebook/KERNEL_STATUS_CHANGE";
    ActionType["RUN"] = "notebook/RUN";
    ActionType["RUN_ALL"] = "notebook/RUN_ALL";
    ActionType["SAVE"] = "notebook/SAVE";
    ActionType["INTERRUPT"] = "notebook/INTERRUPT";
    ActionType["INSERT_ABOVE"] = "notebook/INSERT_ABOVE";
    ActionType["INSERT_BELOW"] = "notebook/INSERT_BELOW";
    ActionType["DELETE"] = "notebook/DELETE";
    ActionType["PORTAL"] = "notebook/PORTAL";
    ActionType["SET_PORTALS"] = "notebook/SET_PORTALS";
})(ActionType || (ActionType = {}));
const actionCreator = actionCreatorFactory('jupyterReact');
export const notebookActions = {
    update: actionCreator(ActionType.UPDATE),
    notebookChange: actionCreator(ActionType.NOTEBOOK_CHANGE),
    activeCellChange: actionCreator(ActionType.ACTIVE_CELL_CHANGE),
    kernelStatusChanged: actionCreator(ActionType.KERNEL_STATUS_CHANGE),
    run: actionCreator(ActionType.RUN),
    save: actionCreator(ActionType.SAVE),
    runAll: actionCreator(ActionType.RUN_ALL),
    interrupt: actionCreator(ActionType.INTERRUPT),
    portal: actionCreator(ActionType.PORTAL),
    setPortals: actionCreator(ActionType.SET_PORTALS),
    insertAbove: actionCreator.async(ActionType.INSERT_ABOVE),
    insertBelow: actionCreator.async(ActionType.INSERT_BELOW),
    delete: actionCreator.async(ActionType.DELETE),
};
/* Epics */
const insertAboveEpic = (action$, state$) => action$.pipe(ofAction(notebookActions.insertAbove.started), tap(action => {
    var _a;
    (_a = state$.value.notebook.adapter) === null || _a === void 0 ? void 0 : _a.commands.execute(cmdIds.insertAbove);
}), ignoreElements());
const insertBelowEpic = (action$, state$) => action$.pipe(ofAction(notebookActions.insertBelow.started), tap(action => {
    var _a;
    (_a = state$.value.notebook.adapter) === null || _a === void 0 ? void 0 : _a.commands.execute(cmdIds.insertBelow);
}), ignoreElements());
const deleteEpic = (action$, state$) => action$.pipe(ofAction(notebookActions.delete.started), tap(action => {
    var _a;
    (_a = state$.value.notebook.adapter) === null || _a === void 0 ? void 0 : _a.commands.execute(cmdIds.deleteCells);
}), ignoreElements());
export const notebookEpics = combineEpics(insertAboveEpic, insertBelowEpic, deleteEpic);
/* Reducers */
export const notebookReducer = reducerWithInitialState(notebookInitialState)
    .case(notebookActions.update, (state, update) => {
    return Object.assign(Object.assign({}, state), update);
})
    .case(notebookActions.run, (state, _) => {
    state.adapter && state.adapter.commands.execute(cmdIds.run);
    return state;
})
    .case(notebookActions.runAll, (state, run) => {
    state.adapter && state.adapter.commands.execute(cmdIds.runAll);
    return state;
})
    .case(notebookActions.interrupt, (state, _) => {
    state.adapter && state.adapter.commands.execute(cmdIds.interrupt);
    return state;
})
    .case(notebookActions.save, (state, _) => {
    state.adapter && state.adapter.commands.execute(cmdIds.save);
    return state;
})
    .case(notebookActions.activeCellChange, (state, activeCell) => {
    return Object.assign(Object.assign({}, state), { activeCell });
})
    .case(notebookActions.notebookChange, (state, notebookChange) => {
    return Object.assign(Object.assign({}, state), { notebookChange });
})
    .case(notebookActions.kernelStatusChanged, (state, kernelStatus) => {
    return Object.assign(Object.assign({}, state), { kernelStatus });
})
    .case(notebookActions.portal, (state, portal) => {
    const portals = state.portals.concat(portal);
    return Object.assign(Object.assign({}, state), { portals });
})
    .case(notebookActions.setPortals, (state, portals) => {
    return Object.assign(Object.assign({}, state), { portals });
})
    .case(notebookActions.insertAbove.done, (state, success) => {
    //    state.adapter && state.adapter.commands.execute(cmdIds.insertAbove);
    return state;
})
    .case(notebookActions.insertBelow.done, (state, success) => {
    //    state.adapter && state.adapter.commands.execute(cmdIds.insertBelow);
    return state;
})
    .case(notebookActions.delete.done, (state, success) => {
    //    state.adapter && state.adapter.commands.execute(cmdIds.deleteCells);
    return state;
});
