import { jsx as _jsx, jsxs as _jsxs } from "@emotion/react/jsx-runtime";
import { useState, useMemo, useEffect } from 'react';
import { OutputAreaModel } from '@jupyterlab/outputarea';
import OutputAdapter from './OutputAdapter';
import CodeMirrorEditor from '../editor/CodeMirrorEditor';
import LuminoAttached from '../../jupyter/lumino/LuminoAttached';
const Output = (props) => {
    const { initialOutput, autoRun, code, kernel, showEditor, executeTrigger, clearTrigger } = props;
    const [model] = useState(new OutputAreaModel({
        trusted: true,
        values: initialOutput,
    }));
    const Output = useMemo(() => {
        return new OutputAdapter(kernel.getJupyterKernel(), model, {});
    }, []);
    useEffect(() => {
        if (!showEditor && autoRun) {
            Output.execute(code);
        }
    }, [executeTrigger]);
    useEffect(() => {
        if (showEditor) {
            Output.execute(code);
        }
    }, [executeTrigger]);
    useEffect(() => {
        if (showEditor) {
            Output.clearOutput();
        }
    }, [clearTrigger]);
    return _jsxs("div", { children: [_jsx("div", { children: showEditor &&
                    _jsx(CodeMirrorEditor, { autoRun: autoRun, code: code, outputAdapter: Output }, void 0) }, void 0),
            _jsx("div", Object.assign({ css: {
                    '& .jp-OutputPrompt': {
                        display: 'none',
                    },
                } }, { children: _jsx(LuminoAttached, { children: Output.panel }, void 0) }), void 0)] }, void 0);
};
Output.defaultProps = {
    initialOutput: [
        {
            "output_type": "execute_result",
            "data": {
                "text/html": [
                    "<h3>I am the default output...  👉  Run the cell to update me...  👀</h3>"
                ]
            },
            "execution_count": 0,
            "metadata": {},
        }
    ],
    executeTrigger: 0,
    clearTrigger: 0,
};
export default Output;
