import { useSelector } from "react-redux";
import actionCreatorFactory from "typescript-fsa";
import { combineEpics } from "redux-observable";
import { delay, map, tap, ignoreElements } from "rxjs/operators";
import { ofAction } from "@datalayer/typescript-fsa-redux-observable";
import { reducerWithInitialState } from "typescript-fsa-reducers";
export const initInitialState = {
    init: 0
};
/* Selectors */
export const selectInit = () => useSelector((state) => {
    return state.initReducer.init;
});
/* Actions */
export var ActionType;
(function (ActionType) {
    ActionType["INCREMENT"] = "init/INCREMENT";
    ActionType["DECREMENT"] = "init/DECREMENT";
})(ActionType || (ActionType = {}));
const actionCreator = actionCreatorFactory('jupyterReact');
export const initActions = {
    increment: actionCreator.async(ActionType.INCREMENT),
    decrement: actionCreator.async(ActionType.DECREMENT)
};
/* Epics */
const initIncrementEpic = action$ => action$.pipe(ofAction(initActions.increment.started), delay(2000), map(action => initActions.increment.done({
    params: action.payload,
    result: undefined
})));
const initDecrementEpic = action$ => {
    return action$.pipe(ofAction(initActions.decrement.started), delay(100), map(action => initActions.decrement.done({
        params: action.payload,
        result: undefined
    })));
};
const loggingEpic = action$ => action$
    .pipe(ofAction(initActions.decrement.started), tap(action => console.log(action.type)), ignoreElements()).pipe(ofAction(initActions.increment.started), tap(action => console.log(action.type)), ignoreElements());
export const initEpics = combineEpics(initIncrementEpic, initDecrementEpic, loggingEpic);
/* Reducers */
export const initReducer = reducerWithInitialState(initInitialState)
    .case(initActions.increment.done, (state) => (Object.assign(Object.assign({}, state), { init: state.init + 1 })))
    .case(initActions.decrement.done, (state) => {
    return Object.assign(Object.assign({}, state), { init: state.init - 1 });
});
export const initReducers = {
    initReducer: initReducer
};
